@@ -1,10 +1,7 @@
 """Abstract base models used by the page management application."""
-from __future__ import unicode_literals
-
-import six
 from django.db import models
 from django.shortcuts import render
-from django.utils.encoding import python_2_unicode_compatible
+from django.utils.crypto import constant_time_compare, salted_hmac
 from watson.search import SearchAdapter
 
 from cms.apps.media.models import ImageRefField
@@ -12,6 +9,29 @@ from cms.models.managers import (OnlineBaseManager, PageBaseManager,
                                  PublishedBaseManager, SearchMetaBaseManager)
 
 
+class PathTokenGenerator:
+    '''
+        A simple token generator that takes a path and generates a hash for it.
+        Intended for use by the CMS publication middleware and OnlineBase derivatives.
+        In reality it just takes a string so it can be used for other purposes.
+    '''
+    key_salt = "django.contrib.auth.tokens.PasswordResetTokenGenerator"
+
+    def make_token(self, path):
+        return salted_hmac(
+            self.key_salt,
+            path,
+        ).hexdigest()[::2]
+
+    def check_token(self, token, path):
+        return constant_time_compare(
+            token,
+            salted_hmac(self.key_salt, path).hexdigest()[::2]
+        )
+
+path_token_generator = PathTokenGenerator()
+
+
 class PublishedBase(models.Model):
 
     """A model with publication controls."""
@@ -44,6 +64,12 @@ class OnlineBase(PublishedBase):
         ),
     )
 
+    def get_preview_url(self):
+        if not hasattr(self, 'get_absolute_url'):
+            return None
+
+        return f'{self.get_absolute_url()}?preview={path_token_generator.make_token(self.get_absolute_url())}'
+
     class Meta:
         abstract = True
 
@@ -217,7 +243,7 @@ class SearchMetaBase(OnlineBase):
 
     def get_context_data(self):
         """Returns the SEO context data for this page."""
-        title = six.text_type(self)
+        title = str(self)
         # Return the context.
         return {
             "meta_description": self.meta_description,
@@ -255,12 +281,9 @@ class SearchMetaBaseSearchAdapter(OnlineBaseSearchAdapter):
 
     def get_live_queryset(self):
         """Selects only live models."""
-        return super(SearchMetaBaseSearchAdapter, self).get_live_queryset().filter(
-            robots_index=True,
-        )
+        return super().get_live_queryset().filter(robots_index=True)
 
 
-@python_2_unicode_compatible
 class PageBase(SearchMetaBase):
 
     """
@@ -273,7 +296,8 @@ class PageBase(SearchMetaBase):
     # Base fields.
 
     slug = models.SlugField(
-        help_text='A user friendly URL'
+        help_text='A unique portion of the URL that is used to identify this '
+                  'specific page using human-readable keywords (e.g., about-us)'
     )
 
     title = models.CharField(
@@ -295,7 +319,7 @@ class PageBase(SearchMetaBase):
 
     def get_context_data(self):
         """Returns the SEO context data for this page."""
-        context_data = super(PageBase, self).get_context_data()
+        context_data = super().get_context_data()
         context_data.update({
             "title": self.browser_title or self.title,
             "header": self.title,
